package net.knarcraft.minstrel;

import org.bukkit.Bukkit;
import org.bukkit.entity.Player;

import java.util.ArrayList;
import java.util.List;

/**
 * A representation of a playlist containing songs
 */
public class Playlist {

    private final List<Song> songs;
    private final boolean loop;
    private int currentlyPlaying = 0;
    private int schedulerId = 0;

    /**
     * Instantiates a new playlist
     *
     * @param songs <p>The songs contained in this playlist</p>
     * @param loop  <p>Whether to loop around when this playlist finishes</p>
     */
    public Playlist(List<Song> songs, boolean loop) {
        this.songs = new ArrayList<>(songs);
        this.loop = loop;
    }

    /**
     * Gets a copy of the songs in this playlist
     *
     * @return <p>The songs in this playlist</p>
     */
    public List<Song> getSongs() {
        return new ArrayList<>(this.songs);
    }

    /**
     * Adds a new song to this playlist
     *
     * @param song <p>The song to add</p>
     */
    public void addSong(Song song) {
        this.songs.add(song);
    }

    /**
     * Removes a song from this playlist
     *
     * @param index <p>The index of the song to remove</p>
     */
    public void removeSong(int index) {
        this.songs.remove(index);
    }

    /**
     * Plays the current song for the given player
     *
     * <p>This should only be used if a player recently joined, or for some other reason cannot currently hear the
     * music.</p>
     *
     * @param trait  <p>The the minstrel to play this playlist for</p>
     * @param player <p>The player to play to</p>
     */
    public void play(MinstrelTrait trait, Player player) {
        //If this playlist is empty, do nothing
        if (this.songs.size() < 1) {
            return;
        }

        for (Song song : this.songs) {
            song.stop(player);
        }

        Song currentSong = this.songs.get(this.currentlyPlaying - 1);
        currentSong.play(trait, player, trait.getVolume(), trait.getPitch());
    }

    /**
     * Plays the next song in this playlist
     *
     * @param trait <p>The minstrel to play this playlist for</p>
     */
    public void play(MinstrelTrait trait) {
        //If this playlist is empty, do nothing
        if (this.songs.size() < 1) {
            return;
        }

        if (this.currentlyPlaying >= songs.size()) {
            if (this.loop) {
                this.currentlyPlaying = 0;
            } else {
                return;
            }
        }

        for (Player player : Bukkit.getOnlinePlayers()) {
            for (Song song : this.songs) {
                song.stop(player);
            }
        }

        Song currentSong = this.songs.get(this.currentlyPlaying);
        currentSong.play(trait, trait.getVolume(), trait.getPitch());
        currentlyPlaying++;
        schedulerId = Bukkit.getScheduler().scheduleSyncRepeatingTask(MinstrelPlugin.getInstance(), () -> {
            if (!currentSong.isPlaying()) {
                Bukkit.getScheduler().cancelTask(schedulerId);
                play(trait);
            }
        }, currentSong.getDuration(), 20);
    }

}
